% REDUCE script @(#)area.red   3.64,  
% copyright 6 June 1991, Wilfrid S. Kendall. 
 
OFF ECHO $  
IN "itovsn3.red", "brown.red", "itosde.red"$ 
 
% Proving the Levy formula for the distribution of 
% stochastic area using stochastic calculus. 
% Full details and references to the originators of 
% this approach may be found in P.Protter (1990),  
% Stochastic Integration and Differential Equations,  
% Springer--Verlag, Berlin, Theorem 42 on page 82 
 
WRITE " 
First introduce a two--dimensional Brownian motion  (X,Y) 
begun at zero, and the corresponding stochastic area  A  
solving the stochastic differential equation 
 
	dA = X*dY-Y*dX 
"$ 
 
BrownianBasis({X,Y},{0,0})$ 
Itosde(A,(dA=X*dY-Y*dX),0)$ 
 
WRITE " 
Now consider a (complex--valued) semimartingale  V  defined using  X 
and  Y  and two deterministic quantities  alfa  and  beta  which are 
twice--continuously--differentiable functions of  time. 
 
NOTE that  I  stands for a square root of  -1,  satisfying  I**2+1=0. 
"$ 
 
DEPEND alfa, time$ DEPEND alfa!', time$ 
DEPEND beta, time$ DEPEND beta!', time$ 
LET 
   DF(alfa,time)   = alfa!',   DF(beta,time)   = beta!', 
   DF(alfa!',time) = 0,        DF(beta!',time) = 0 
$ 
V := I*u*A - (alfa/2)*(X**2+Y**2)+beta ; 
 
WRITE " 
We consider when  EXP(V)  might be a local martingale by computing 
 
	<< Drift d EXP(V) >>/EXP(V) 
 
NOTE use of escape character ! to create a primed identifier, for 
example in  beta!'  which REDUCE reads as beta'. 
"$ 
 
criterion := << Drift d EXP(V) >>/EXP(V) ; 
 
WRITE "Tidy this up using FACTOR switch."$ 
 
ON FACTOR; criterion; OFF FACTOR; 
 
WRITE " 
It will be a local martingale precisely when the following 
two quantities vanish! 
"$ 
 
e1 := COEFFN(criterion,X,2); 
e2 := SUB(X=0,Y=0,criterion); 
 
WRITE " 
In principle we should now invoke a solver of systems of first--order 
differential equations! However it is easy to see if  e1  and  e2  
vanish and if we stipulate that  alfa  and  beta  both vanish at 
time  t0  then  alfa  and  beta  are as follows: 
"$ 
 
beta!_try := - LOG(COSH(u*(t0-time))); 
alfa!_try :=   DF(beta!_try,time); 
 
WRITE " 
First check we do have solutions: 
"$ 
 
subs := {alfa=alfa!_try, 
         beta=beta!_try, 
         alfa!'=<<DF(alfa!_try,time)>>, 
         beta!'=<<DF(beta!_try,time)>>} $ 
 
WRITE " 
Expression  e1  should yield zero under substitution: " $  
e1 = SUB(subs,e1); 
 
WRITE " 
Expression  e2  should yield zero under substitution: " $ 
e2 = SUB(subs,e2); 
 
WRITE " 
As a final check we consider the semimartingale  V  with 
the substitutions performed:  
"$ 
trial := EXP(SUB(subs,V)); 
 
WRITE " 
A somewhat lengthy calculation calculates drift as ", Drift d trial $ 
 
WRITE " 
Consider that 
 
	V = I*u*A - (alfa/2)*(X**2+Y**2)+beta  
 
and  A,  X,  Y  start at zero. Furthermore boundedness considerations 
allow us to deduce that   EXP(V)  is actually a true martingale. 
We deduce the characteristic function of  A  at time  t0  is given by 
 
	EXP(beta(0)) = 
"$ 
Initial!_Value(0, SUB(subs,EXP(V)) ); 
 
WRITE" 
and Levy's formula for the distribution of the stochastic area 
now follows. 
"$ 
 
WRITE " 
	End of output from  ex/area.red. 
	================================ 
"$ 
 
$END$ 
 
