% REDUCE script @(#)escape.red   3.64,  
% copyright 6 June 1991, Wilfrid S. Kendall. 
OFF ECHO $  
WRITE " 
      An exercise examining power-law rates of escape for Bessel 
      ========================================================== 
      processes of (possibly fractional) dimension exceeding two. 
      =========================================================== 
 
The results below undergird work by Pei Hsu and Wilfrid S. Kendall on 
``Limiting angle of Brownian motion in certain two-dimensional 
Cartan-Hadamard manifolds'' (Warwick University Department of 
Statistics Research Report 216). The results were originally obtained 
(with more besides, including integral tests) by: 
 
      M.Motoo (1959), ``Proof of the law of iterated logarithm through 
      diffusion equation.'' Ann. Inst. Statist. Math. 10, 21--28. 
 
      T.Shiga and S.Watanabe (1973), ``Bessel Diffusions as a One-Parameter 
      Family of Diffusion Processes.'' Zeitschrift fur  
      Wahrscheinlichkeitstheorie und v. Geb. 27, 37--46. 
 
See also section 4.12 of 
 
      K.Ito and H.P.McKean (1974), ``Diffusion Processes and their Sample 
      Paths.'' Second printing (corrected), Springer-Verlag, Berlin. 
 
The treatment below is also described in my research report ``Symbolic 
Ito calculus: an introduction'' (Warwick University Department of 
Statistics Research Report 217). 
 
Warwick University Statistics Research Reports may be obtained by 
writing to the address below and requesting a copy: 
 
	The Secretary, Statistics Department, 
	University of Warwick, COVENTRY CV4 7AL, UK. 
 
"$ 
IN "itovsn3.red", "feller.red" $ 
 
WRITE " 
Introduce the Bessel process  R  of fractional dimension  a  and its 
stochastic differential  dR."$ 
Reset!_Ito() $ Introduce(R, dR) $ 
LET dR**2 = dt $ Add!_Drift(dR, (a-1)*dt/(2*R) ) $ 
 
WRITE "-------------------------------- 
 
1. Diffusion analysis of Bessel process of fractional dimension  a. 
=================================================================== 
 
For an exercise we compute the diffusion coefficients (the drift  a1,  and   
a2  the infinitesimal variance) for the diffusion  R."$ 
cc := diffusion!_coefficients(R, dt) $ 
cc := FOR EACH elt IN cc COLLECT SUB(R=x, elt)$ 
a2 := FIRST cc $ a1 := SECOND cc $ DEPEND y,x$ 
WRITE " 
Write down the differential equations for 
(a)   the infinitesimal generator: "$ 
inf!_generator(y,a2,a1,x)=0; 
WRITE " 
(b)   the formal adjoint of the infinitesimal generator: "$ 
formal!_adjoint(y,a2,a1,x)=0; 
WRITE " 
and compute the gradient  rho  of the natural scale of the diffusion: 
this allows expression of the generator in divergence form  
     (a2 * rho**2) d/(rho * dx) d/(rho * dx), "$ 
rho := scale!_gradient(a2,a1,x); 
WRITE " 
Now find the scale function, which is the non-constant harmonic function. 
It is given by: "$ 
scale!_function := scale!_function(a2,a1,x); 
WRITE " 
Hence the general harmonic function is given by:"$ 
general!_harmonic := c0+c1*scale!_function(a2,a1,x); 
WRITE " 
Verify that the general harmonic is annihilated by the generator: "$ 
inf!_generator(y,a2,a1,x) = inf!_generator(general!_harmonic,a2,a1,x); 
 
WRITE " 
Because the scale function integrates to an unbounded function  
(for  a > 2)  we know that  R  is recurrent. "$ 
 
WRITE " 
Compute the density of the speed measure, an invariant density for the  
diffusion over the positive half-axis."$ 
density := speed!_density(a2,a1,x); 
 
WRITE " 
Verify this density solves the formal adjoint equation:"$ 
formal!_adjoint(y,a2,a1,x) = 
formal!_adjoint(density,a2,a1,x); 
 
WRITE " 
Because the invariant density is integrable over the range of the 
recurrent diffusion  R  we know that  R  is positive--recurrent. 
 
CONCLUSION: In the original time--scale the process  R  is a  
positive--recurrent diffusion on the positive half--line. 
 
"$ 
 
WRITE "-------------------------------- 
 
2. Diffusion analysis for  X=R/SQRT(time)  in new time-scale  dt/time. 
====================================================================== 
  
   (NB: Ito and McKean consider  R**2/time  instead! ... )"$ 
cc := diffusion!_coefficients(R/SQRT(time), dt/time) $ 
cc := FOR EACH elt IN cc COLLECT SUB(SOLVE(x=R/SQRT(time),R), elt)$ 
a2 := FIRST  cc $ a1 := SECOND cc $ DEPEND y,x$ 
WRITE " 
The differential equations for 
(a)   the infinitesimal generator: "$ 
inf!_generator(y,a2,a1,x)=0; 
WRITE " 
(b)   the formal adjoint of the infinitesimal generator: "$ 
formal!_adjoint(y,a2,a1,x)=0; 
WRITE " 
The gradient  rho  of the natural scale of the diffusion:"$ 
rho := scale!_gradient(a2,a1,x); 
 
WRITE " 
The scale function is given by: "$ 
scale!_function := scale!_function(a2,a1,x); 
WRITE" 
Verification that the scale function is annihilated by the generator: "$ 
inf!_generator(y,a2,a1,x) = inf!_generator(scale!_function,a2,a1,x); 
 
WRITE " 
Because the scale function integrates to an unbounded function  
(for  a > 2)  we know that  X  is recurrent. "$ 
 
WRITE " 
The density of the speed measure, an invariant density for the diffusion  X 
over the positive half-axis. Note this is integrable over (0,oo)   
(if  a > 2 )  so the new diffusion is positive-recurrent. 
FIRST FORM: "$ 
density := speed!_density(a2,a1,x); 
WRITE " 
SECOND (EQUIVALENT) FORM: "$ 
density := x**(a-1)*EXP(-x**2/2); 
WRITE " 
Verification that these forms solve the formal adjoint equation: 
FIRST FORM: "$ 
formal!_adjoint(y,a2,a1,x) = 
formal!_adjoint(speed!_density(a2,a1,x),a2,a1,x); 
WRITE " 
SECOND FORM: "$ 
formal!_adjoint(y,a2,a1,x) = formal!_adjoint(density,a2,a1,x); 
WRITE " 
CONCLUSION: In the new time--scale  dt/time  the process  R**2/time 
is a positive--recurrent diffusion on the positive half--line.  
 
"$ 
 
WRITE "-------------------------------- 
 
Now consider power laws for rate of escape. Does  R  hit the barrier 
given by  SQRT(time)*(1+eta)  for arbitrarily large  time? 
 
3. Analysis of  R/SQRT(time**(1+eta)). 
====================================== 
 
First consider the case of positive  eta.  Consider a NEW semimartingale  
X = R/SQRT(time**(1+eta))  in the time-scale  dt/(time**(1+eta)).  "$  
X := R/SQRT(time**(1+eta)) $ dtau := dt/(time**(1+eta))$  
WRITE "  
Infinitesimal variance and drift coefficients are given (in terms of  R and  
the original time-scale) by: "$  
sd2 := <<(d X)**2>>/dtau; % NOTE use of <<>> to force second-order structure! 
Esd := <<Drift d X>>/dtau; 
 
WRITE " 
For positive  eta  the crucial issue is a balance between the behaviour of 
the drift term and the new time-scale. Here is an expression of the drift in 
terms of the new process  X,  using  time=tt**2  to simplify presentation,"$ 
CLEAR x$ 
Esd := SUB(SOLVE(x=R/SQRT(time**(1+eta)),R),Esd)$ 
Esd := SUB(time=tt**2,Esd); 
WRITE " 
By comparison of stochastic differential equations it follows 
that the process under consideration in the new time--scale 
is bounded above by an ordinary Bessel process of fractional  
dimension  a. 
"$ 
 
WRITE " 
On the other hand the integral of dtau/dt with respect to time is  
convergent, "$ 
INT(dtau/dt,time); 
WRITE " 
and so the  tau  time--scale is bounded. "$ 
 
WRITE " 
Hence the process under consideration has finite limes supremum. 
However by the 0:1 law for the positive--recurrent diffusion 
R/SQRT(time)  it follows this limes supremum is also non--random 
and does not depend on the initial value of the process. 
By Bessel process scaling arguments the limes supremum must be zero. 
 
CONCLUSION: The process under consideration must converge to zero, 
so the Bessel process is not certain to keep on hitting boundaries  
of the form  SQRT(time)**(1+eta)  for  eta > 0. 
 
 
"$ 
 
WRITE "-------------------------------- 
 
IMPORTANT: In the following analysis  a > 2. 
 
4. Analysis of  R/SQRT(time**(1-eta)). 
====================================== 
 
Now consider the case of negative  eta.  Here it is convenient to consider 
the renormalized diffusion  X=R/SQRT(time)  in natural scale 
tau  (such that  dtau=dt/time).  The barrier becomes   
X = EXP( - eta*tau/2 )  in this new time-scale.  
 
"$ 
 
WRITE " 
We search for a function  f(X,tau)  such that 
	(a)	f(X,tau)  is a supermartingale, 
	(b)	f(X,tau)  is decreasing in  X   
			whenever  X > EXP( - eta*tau/2 ), 
	(c)	f(EXP( - eta*tau/2 ),tau)  is non--decreasing  
			in  tau  for all sufficiently large  tau. 
"$ 
 
WRITE " 
First construct the scale function and an approximate scale function 
for  X.  The scale function is determined by its derivative. The 
approximation is given exactly. 
"$ 
 
CLEAR X, x $ 
OPERATOR scalefn, scalefn!_asymp $ 
FOR ALL x,a LET DF(scalefn(x,a),x)  = EXP(x**2/2)*x**(1-a), 
                scalefn!_asymp(x,a) = x**(2-a)/(a-2) $ 
WRITE " 
Exact scale function has derivative ", DF(scalefn(x,a),x) $ 
WRITE " 
Approximation is given by ", scalefn!_asymp(x,a) $ 
WRITE " 
This is how they are affected by the infinitesimal generator 
(constructed in section 2 above):" $ 
inf!_generator(y,a2,a1,x) = inf!_generator(scalefn(x,a),a2,a1,x); 
inf!_generator(y,a2,a1,x) = inf!_generator(scalefn!_asymp(x,a),a2,a1,x); 
 
WRITE " 
We consider a function  defined for  x <= 1  and for  2 < b < a  by ", 
work := x+scalefn!_asymp(x,b)+k $ 
WRITE " 
and defined to be constant for  x >= 1  so ", 
workat1 := SUB(x=1,work) $ 
WRITE " 
Note that the function is once continuously differentiable 
everywhere, and twice continuously differentiable save at  x = 1."$ 
 
WRITE " 
Consider the action of the infinitesimal generator  L,  when  x < 1 . 
We find  (L work)/work  is given by:"$ 
 
ON FACTOR$ ratio:=inf!_generator(work,a2,a1,x)/work;  
 
WRITE " 
For small  x  the dominant terms produce ", SUB(x**b=0,ratio) $ 
WRITE " 
and this is negative for sufficiently small  x  (recall  2 < b < a)"$ 
 
WRITE " 
It follows that for sufficiently large  k  the ratio above is 
bounded above by  epsilon  for  0 < x < 1.  Hence the following  
defines a supermartingale as required in (a) above:"$ 
 
f := (b-2)*EXP(-epsilon*tau)*work; 
 
 
WRITE " 
Property (b) is apparent immediately, since for fixed  tau 
the function  f  is constant for  x >= 1  and for  x < 1  
has derivative ", 
DF(f,x) $ 
 
WRITE " 
For property (c) consider  f  over the curve  x = EXP( - eta*tau/2 ). 
We parametrize by  x  so that the curve is given by 
EXP(-epsilon*tau) = x**c  where  c = 2*epsilon/eta.  Along this 
curve we must consider  
", 
workcurve := SUB(E**(epsilon*tau) = 1/x**c, f), 
" 
and show it is DECREASING in  x  for sufficiently small  x 
(since  x  is decreasing in  tau  along the curve ...). The  
derivative is given by  
"$ 
DF(workcurve,x); 
 
WRITE " 
When  x  is sufficiently small the dominant terms yield 
a numerator of: ", 
SUB(x**b=0,NUM DF(workcurve,x)) $ 
 
WRITE " 
Hence (c) is satisfied if  b - 2 > c  (and we may obtain this 
by choosing  epsilon  sufficiently small, since  c = 2*epsilon/eta."$ 
 
WRITE " 
CONCLUSION: The original Bessel process will not keep on hitting 
boundaries of the form  SQRT(time)**(1-eta)  for  eta > 0. 
 
 
"$ 
OFF ECHO$ 
WRITE " 
      End of output from  ex/escape.red. 
      ================================== 
"$ 
 
END$ 
 
