% REDUCE script @(#)geom.red   3.64, 
% copyright 6 June 1991, Wilfrid S. Kendall. 
 
OFF ECHO$ 
IN "itovsn3.red", "riem.red"        $ 
WRITE " 
A number of examples of automatic geometry! For brief descriptions see 
 
	Kendall, W.S. (1990) ``Computer Algebra and Stochastic Calculus 
	'' (Warwick University Department of Statistics Research Report 
	203) Notices Amer. Math. Soc. Vol 37, 1254--1256 (1990). 
 
	Kendall, W.S. (1991) ``Symbolic Ito calculus: an introduction'' 
	(Warwick University Department of Statistics Research Report 
	217) To appear in Proceedings of Conference on ``Numerical 
	probabilities'', Luminy, Marseilles, March 1991. 
 
"$ 
 
% ------------------------------------------------ 
% 
WRITE "Brownian motion on the plane."$ 
 
%WRITE "In the next step and in following, ignore error messages 
%*** Fixed(0,B) not found  
%et cetera."$ 
 
Reset!_Ito( ) $ 
 
Introduce(A, dA) $ Introduce(B, dB) $ 
 
LET dA**2 = 4 * A * dt, 
 dB**2 = dt, dB * dA = 0 $ 
 
Add!_Drift(dA, dt) $ 
Add!_Drift(dB, 0) $ 
 
Fixed(0,A) := 0 $ 
Fixed(0,B) := 0 $ 
 
Compute!_Geometry( 
   "Brownian motion on the 2-plane", 
      {A, B}) $ 
WRITE "NOTE: No Ricci curvatures reported because we have set ON NERO 
and the curvatures are all zero!" $ 
 
WRITE "The invariant ", 
  density := Compute!_density( ) $ 
% ------------------------------------------------ 
 
$PAUSE$ 
 
% ------------------------------------------------ 
% 
WRITE "Brownian motion on the 2-sphere."$ 
 
Reset!_Ito( ) $ 
 
% Trigonometrical relationship needed to simplify geometry later: 
 
FOR ALL u LET SIN(u) * COT(u) = COS(u) $ 
 
% Spherical Brownian motion (R,U): 
% in polar coordinates. 
 
CLEAR R, dR, U, dU $ 
Introduce(R, dR) $ Introduce(U, dU) $ 
 
LET dR**2 = dt, dU**2 = dt / SIN(R)**2, 
 dR * dU = 0 $ 
 
Add!_Drift(dR, COT(R) * dt / 2) $  
Add!_Drift(dU, 0) $ 
 
Fixed(0,R) := 0 $ 
Fixed(0,U) := 0 $ 
 
Compute!_Geometry( 
  "Brownian motion on the 2-sphere", 
      {R, U})      $ 
WRITE "The invariant ", 
  density := Compute!_density( ) $ 
% ------------------------------------------------ 
 
$PAUSE$ 
 
% ------------------------------------------------ 
% 
WRITE"Brownian motion on the hyperbolic plane."$ 
 
Reset!_Ito( ) $ 
 
% Hyperbolic relationship needed to simplify geometry later: 
 
FOR ALL u LET SINH(u) * COTH(u) = COSH(u) $ 
 
% Hyperbolic Brownian motion (R,U): 
% in polar coordinates. 
 
CLEAR R,dR,U,dU $ 
Introduce(R,dR) $ Introduce(U,dU) $ 
 
LET dR**2 = dt, dU**2 = dt / SINH(R)**2, 
 dR * dU = 0 $ 
 
Add!_Drift(dR, COTH(R) * dt / 2) $  
Add!_Drift(dU, 0) $ 
 
Fixed(0,R) := 0 $ 
Fixed(0,U) := 0 $ 
 
Compute!_Geometry( 
   "Brownian motion on the hyperbolic plane.", 
      {R, U}) $ 
WRITE "The invariant ", 
  density := Compute!_density( ) $ 
% ------------------------------------------------ 
 
$PAUSE$ 
% ------------------------------------------------ 
WRITE" 
Shape diffusion of three points in n-space (using a form 
obtained before identifying the underlying spherical geometry): 
"$ 
 
Reset!_Ito( ) $ 
 
% Trigonometrical relationship needed to simplify geometry later: 
 
FOR ALL u LET SIN(u) * COT(u) = COS(u) $ 
 
% Three-point shape diffusion: 
 
CLEAR R, dR, U, dU $ 
Introduce(R, dR) $ Introduce(U, dU) $ 
 
LET dR**2 = 16 * dt * R * (1 - R), 
 dR * dU = 0, dU**2 = 4 * dt / R $ 
 
Add!_Drift(dR, 8 * dt * (1 - (n+1) * R / 2)) $ 
Add!_Drift(dU, 0) $ 
 
Fixed(0,R) := 0 $ 
Fixed(0,U) := 0 $ 
 
Compute!_Geometry( 
   "Shape diffusion of three points in n-space.", 
      {R,U})$ 
WRITE "The invariant ", 
  density := Compute!_density( ) $ 
WRITE " 
NOTE: Ignore complex constant factors!"$ 
% ------------------------------------------------ 
 
$PAUSE$ 
 
% ------------------------------------------------ 
WRITE" 
Shape diffusion of three points in 3-space (using another form 
obtained before identifying the underlying spherical geometry): 
"$ 
WRITE " 
(NOTE: this approach could also be used for general n-space.)"$ 
 
Reset!_Ito( ) $ 
 
% Three-point shape diffusion on 3-space: 
 
CLEAR A, dA, B, dB $ 
Introduce(A, dA) $ Introduce(B, dB) $  
C := 1 - A - B $ 
 
LET dA**2 = 4 * dt * A * (2 - 3 * A) / 3,  
 dB**2 = 4 * dt * B * (2 - 3 * B) / 3,  
 dA*dB = 2 * dt * ( 2 * (A-B)**2 - A**2 - B**2 - C**2 ) / 3$ 
 
Add!_Drift(dA, 2 * dt * (1 - 3 * A)) $ 
Add!_Drift(dB, 2 * dt * (1 - 3 * B)) $ 
 
Fixed(0,A) := 0 $ 
Fixed(0,B) := 0 $ 
 
Debug := 0 $ 
Compute!_Geometry( 
   "Shape diffusion of three points in 3-space.", 
      {A,B})$ 
WRITE "The invariant ", 
  density := Compute!_density( ) $ 
CLEAR Debug $ 
WRITE "NOTE: Ahem! Look carefully at the invariant density! 
It is constant!"$ 
 
% ------------------------------------------------ 
WRITE  
"Don't worry about unknown variables  
in the next bit        (7/3/91)" $ 
$PAUSE$ 
 
% ------------------------------------------------ 
WRITE" 
Shape diffusion of  k  points in n-space (using the 
eigenvalue part only!) for  n=5: 
"$ 
 
Reset!_Ito( ) $ 
 
k := 5 $ CLEAR n $ % FIVE points in  n-space. 
 
CLEAR x, dx $ OPERATOR x, dx $ 
FOR p:=1:k DO  
BEGIN 
 CLEAR << MKID(e, p) >>, << MKID(de, p) >>; 
 x(p) := MKID(e,p); dx(p) := MKID(de,p)  
 END $ 
 
Diffusion!_List := FOR p:=2:k-1 COLLECT x(p) $ 
e1 := 0 $ LET << x(k) >> = 1 - FOR p:=1:k-1 SUM x(p) $ 
 
WRITE " 
The operator  Inv  is the inverse operator:  Inv(x) = 1/x. 
This prevents unhelpful ``simplifications''." $ 
 
OPERATOR Trm, Inv $ 
%FOR p:=1:k DO Trm(p) := 
% FOR q:=1:k SUM  
%  IF q NEQ p THEN 1/(x(q) - x(p)) ELSE 0 $ 
FOR p:=1:k DO Trm(p) := 
 FOR q:=1:k SUM  
  IF q NEQ p THEN Inv(x(q) - x(p)) ELSE 0 $ 
%FOR ALL x LET DF(Inv(x), x) = -1/x**2, 
%              Inv(-x)       = -Inv(x), 
%              Inv(x)*x      =  1      $ 
 
 
IN "diffuse.red" $ 
Diffusion!_Define(Diffusion!_List,  
   FOR p:=2:k-1 COLLECT  
     4 * (1 - x(p)) * x(p) * dt ,  
   FOR p:=2:k-1 COLLECT  
   FOR q:=p+1:k-1 COLLECT  
   - 4 * x(p) * x(q) * dt ,  
   FOR p:=2:k-1 COLLECT  
  ( (2 * Trm(p) + (k-1) * n) * x(p) + k-n ) * dt , 
   FOR p:=2:k-1 COLLECT 1/d ) $ 
 
Debug:=0 $ 
WRITE "The next bit takes a long time" $ 
Compute!_Geometry( 
        "Shape diffusion of  k  points in  n-space.", 
                Diffusion!_List) $ 
CLEAR Debug $ 
WRITE " 
Actually the drift IS rotation free, and the invariant 
density may be calculated if integration rules are added for  
the operator  Inv. 
 
We do this after the PAUSE; be warned that it takes a long time! 
 
"$ 
PAUSE$ 
 
WRITE "The invariant density (up to a complex constant factor!) is:"$ 
Compute!_density(); 
% ------------------------------------------------ 
 
$END$ 
 
