% REDUCE script @(#)hedge.red   3.64,  
% copyright 6 June 1991, Wilfrid S. Kendall.  
 
OFF ECHO$  
WRITE " 
      Example of symbolic Ito calculus applied to a hedging problem. 
      ============================================================== 
 
      See the paper 
	J.D.Duffie and H.R.Richardson (1991), ``Mean-variance hedging 
	in continuous time.'' Ann. Appl. Probab. 1, 1--15. 
"$ 
IN "itovsn3.red", "itosde.red" $ 
                                
WRITE " 
Introduce two correlated Brownian motions,  B  and  xi." $ 
Introduce(B,dB) $ Introduce(xi,dxi) $ 
LET dB**2=dt, dxi**2=dt, dB*dxi=rho*dt $ 
Add!_Drift(dB,0) $ Add!_Drift(dxi,0) $ 
DEPEND rho,time $ 
WRITE " 
Correlation is  rho,  time--varying but deterministic." $ 
 
WRITE " 
The two asset prices  Sprice  and  Fprice  are given by linear 
stochastic differential equations driven by  B  and  xi, 
with time--varying coefficients: 
 
      dSprice = mu*Sprice*dt + si*Sprice*dB 
      dFprice =  m*Fprice*dt + v*Fprice*dxi 
 
" $ 
 
Itosde( Sprice, (dSprice = mu*Sprice*dt + si*Sprice*dB), Sprice0 ) $ 
Itosde( Fprice, (dFprice =  m*Fprice*dt + v*Fprice*dxi), Fprice0 ) $ 
DEPEND mu,time $ DEPEND si,time $ 
DEPEND  m,time $ DEPEND  v,time $ 
 
WRITE " 
Define the ``tracking process''  Z: 
" $ 
gamma := m*si*rho/v - mu $ 
Z := k*EXP(-Total+ItoIntegral(gamma*dt)) * Sprice; 
WRITE " 
NOTE that  Total  here is the integral of  gamma*dt  over the time 
interval  [0,T],  so that  Z  at time  T  equals  k*Sprice. 
"$ 
 
WRITE " 
Investment strategy operator  Phi: 
 
FOR ALL x LET Phi(x) = ((m/v**2)*(L-Z-x) - (si*rho/v)*Z )/Fprice  
 
"$ 
OPERATOR Phi $ 
FOR ALL x LET Phi(x) = 
((m/v**2)*(L-Z-x) - (si*rho/v)*Z )/Fprice $ 
 
Itosde( Gs, (dGs =  Phi(Gs)*dFprice), 0 ) $ 
 
WRITE " 
Introduce an alternative trading strategy: 
 
      dG =  theta*dFprice 
 
" $ 
DEPEND theta, time $ 
Itosde( G, (dG =  theta*dFprice), 0 ) $ 
 
WRITE " 
Now consider an inner--product approach: 
analyze the expectation of 
 
      H = (L-Z-Gs)*G 
 
We do this by considering the ratio 
 
      (Drift d H)/H 
 
after Lemma 2 of Duffie--Richardson 
" $ 
H := (L-Z-Gs)*G $ 
(Drift d H)/H; 
 
WRITE " 
Lemma 2 explains, because  H = 0  at time zero it 
follows that  H = 0 for all time!" $ 
 
WRITE " 
	For a good exercise, try to compute the variance as in the  
	appendix to the Duffie--Richardson paper! 
 
	End of output from  ex/hedge.red. 
	================================= 
"$ 
 
$END$ 
 
