% REDUCE script @(#)riem.red   3.64, 
% copyright 6 June 1991, Wilfrid S. Kendall. 
 
ON COMP $ 
% ------------------------------------------------ 
% Script to compute intrinsic Riemannian geometry given 
% a list  Diffusion!_List  (Diffusion!_List)  of semimartingales 
% comprising an elliptic diffusion with smooth coefficients. 
% One version of Compute!_potential uses a global variable  
% which must be kept clear: Ito!_unique. 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
PROCEDURE Initialize!_tensors(Diffusion!_List)$ 
BEGIN 
        n!_D!_L := LENGTH Diffusion!_List $ 
% 
        MATRIX  R!_g(n!_D!_L,n!_D!_L), R!_ginv(n!_D!_L,n!_D!_L),  
                Ricci!_uu(n!_D!_L,n!_D!_L), Ricci!_dd(n!_D!_L,n!_D!_L), 
                Ricci(n!_D!_L,n!_D!_L), 
                R!_drift(n!_D!_L,1),  
                R!_drift!_d(n!_D!_L,1), R!_drift!_u(n!_D!_L,1),   
                R!_arg(n!_D!_L,1) $ 
% 
        FOR i:=1:n!_D!_L DO R!_arg(i,1) := PART(Diffusion!_List,i) $ 
        END $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% Compute metric and co-metric tensors and co-volume form: 
% 
PROCEDURE Compute!_metric $ 
BEGIN 
        FOR i:=1:n!_D!_L DO BEGIN 
                SCALAR sd $ 
                sd := d R!_arg(i,1) $ 
                R!_drift(i,1) := <<Drift sd>>/dt $ 
                R!_g(i,i) := <<sd ** 2>>/dt $ 
        FOR j:=1:n!_D!_L DO R!_g(i,j) := R!_g(j,i)    % note use of symmetry! 
                        := <<sd * d R!_arg(j,1)>>/dt 
                END $ 
% 
        R!_gvol := DET(R!_g) $ 
        IF R!_gvol = 0  
        THEN <<REDERR "Diffusion ",Diffusion!_List," is singular!">> 
        ELSE R!_gvol := 1/SQRT R!_gvol $ 
% 
        R!_ginv := R!_g ** (-1) $       % The expensive step! 
        END $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% Compute Christoffel symbols. 
% 
PROCEDURE Compute!_Christoffel $ 
BEGIN 
        CLEAR   Gamma!_ddd, Gamma!_ddu, temp $ 
        ARRAY   Gamma!_ddd(n!_D!_L,n!_D!_L,n!_D!_L),  
                Gamma!_ddu(n!_D!_L,n!_D!_L,n!_D!_L),  
                temp(n!_D!_L,n!_D!_L,n!_D!_L) $ 
% 
% Compute Christoffel symbols and (intrinsic) drift: 
% 
FOR i:=1:n!_D!_L DO 
FOR j:=1:n!_D!_L DO 
FOR k:=j:n!_D!_L DO temp(i,j,k) := temp(i,k,j)  % note use of symmetry 
        := DF(R!_ginv(j,k),R!_arg(i,1)) $ 
% 
FOR i:=1:n!_D!_L DO 
FOR j:=i:n!_D!_L DO 
FOR k:=1:n!_D!_L DO Gamma!_ddd(i,j,k) :=        % note use of symmetry 
        Gamma!_ddd(j,i,k) :=                    % (via zero torsion) 
        (temp(i,j,k) + temp(j,i,k) - temp(k,i,j))/2 $ 
FOR i:=1:n!_D!_L DO  
FOR j:=i:n!_D!_L DO 
FOR k:=1:n!_D!_L DO Gamma!_ddu(i,j,k) :=        % note use of symmetry 
        Gamma!_ddu(j,i,k) :=                    % (via zero torsion) 
        FOR a:=1:n!_D!_L SUM Gamma!_ddd(i,j,a)*R!_g(a,k) $ 
% 
        CLEAR temp $ 
        END $ 
% 
% Compute (intrinsic) drift using Christoffel symbols: 
% 
%FOR k:=1:n!_D!_L DO    % Take off the non-intrinsic part of the drift. 
%       R!_drift!_u(k,1) := R!_drift(k,1)     - 
%               - (1/2)*FOR i:=1:n!_D!_L SUM 
%                       FOR j:=1:n!_D!_L SUM 
%                       R!_g(i,j)*Gamma!_ddu(i,j,k) $ 
%R!_drift!_d := R!_ginv*R!_drift!_u $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% Compute (intrinsic) drift using stochastic calculus 
% (returns  1  if drift is gradient of potential, 
% else 0. 
% 
PROCEDURE Compute!_Drift $ 
BEGIN 
        FOR i:=1:n!_D!_L DO R!_drift!_u(i,1) := 
                (1/2)*FOR j:=1:n!_D!_L SUM 
                        DF(R!_gvol*R!_g(i,j),R!_arg(j,1))/R!_gvol $ 
        R!_drift!_u := R!_drift - R!_drift!_u $ 
        R!_drift!_d := R!_ginv*R!_drift!_u $ 
        END $ 
% 
% Now test to see if drift is gradient of potential function 
% (can be computationally expensive): 
%  
PROCEDURE Test!_Potential $  
IF n!_D!_L=1 THEN 1 ELSE Test!_Potential!_1(1,2) $ 
% 
PROCEDURE Test!_Potential!_1(i,j) $ % i < j always! 
IF Test!_Potential!_2(i,j)=0 THEN 0  
ELSE IF n!_D!_L=i+1 THEN 1 
ELSE IF n!_D!_L=j   THEN Test!_Potential!_1(i+1,i+2) 
ELSE Test!_Potential!_1(i,j+1) $ 
% 
PROCEDURE Test!_Potential!_2(i,j) $ % i < j always! 
IF DF(R!_drift!_d(i,1),R!_arg(j,1)) = DF(R!_drift!_d(j,1),R!_arg(i,1))  
THEN 1 ELSE 0 $ 
% 
% Next step may be computationally VERY expensive! 
% WARNING: log(x) means log(|x|).  
% 
PROCEDURE Compute!_potential $ Compute!_potential!_A() $ 
% 
OPERATOR Env $ LINEAR Env $ 
PROCEDURE Inv!_1(x) $ 1/x $ 
PROCEDURE INT!_1(x, dx) $ % Allows specification of integration strategy. 
SUB(Env=INT, SUB(Inv=Inv!_1, Env(x, dx) ) ) $ 
% 
PROCEDURE Compute!_potential!_A $ % Uses Ito!_unique as clear variable. 
2*FOR i:=1:n!_D!_L SUM  
 BEGIN SCALAR integral; 
 integral := INT!_1(integrand, Ito!_unique) 
  WHERE integrand =   
   SUB( (FOR j:=1:n!_D!_L COLLECT R!_arg(j,1)=Ito!_unique*R!_arg(j,1)), 
                R!_drift!_d(i,1) ) * R!_arg(i,1) ;  
 RETURN SUB(Ito!_unique=1, integral) - SUB(Ito!_unique=0, integral) 
 END$ 
% 
PROCEDURE Make!_sbs $ % Will not always be appropriate. 
FOR i:=1:n!_D!_L COLLECT R!_arg(i,1) = MKID(R!_arg(i,1),0) $ 
% 
PROCEDURE Compute!_potential!_B(sbs) $ % Requires substitution list sbs for 
 BEGIN SCALAR work, work1 $            % initial values of the R!_arg 
  work := 0;               % Seems faster than Compute!_potential!_A() 
  FOR i:=1:n!_D!_L DO      % but needs care in building sbs. 
  BEGIN 
   work := work + 
           << work1 := INT!_1(R!_drift!_d(i,1), R!_arg(i,1)); 
              SUB(REST sbs, work1) - SUB(sbs, work1) >>; 
   sbs := REST sbs 
   END; 
  RETURN 2*work 
  END $ 
% 
PROCEDURE Compute!_density $ 
R!_gvol * EXP(Compute!_potential()) $ 
% 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% Now compute Ricci curvature tensor; 
PROCEDURE Compute!_Ricci $ 
BEGIN 
        FOR i:=1:n!_D!_L DO 
        FOR j:=1:n!_D!_L DO 
        Ricci!_dd(i,j) := 
        <<FOR a:=1:n!_D!_L SUM 
                        DF(Gamma!_ddu(i,j,a),R!_arg(a,1)) 
                -       DF(Gamma!_ddu(a,j,a),R!_arg(i,1))>> 
        + 
        <<FOR a:=1:n!_D!_L SUM FOR c:=1:n!_D!_L SUM 
                        Gamma!_ddu(i,j,c)*Gamma!_ddu(a,c,a) 
                -       Gamma!_ddu(a,j,c)*Gamma!_ddu(i,c,a)>> $ 
        RETURN Ricci := R!_g*Ricci!_dd $ 
        END $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% PROCEDURE to report results: 
ALGEBRAIC PROCEDURE Report!_Geometry(label, potential!_flag) $ 
BEGIN 
        WRITE "------------------------------" $ 
        WRITE label $ 
        WRITE "Infinitesimal variance tensor:" $ 
        ON NERO $ WRITE R!_g := <<R!_g>> $ OFF NERO$ 
        WRITE ""$ 
        WRITE  
"Intrinsic drift using inverse variance tensor to yield metric:" $ 
        WRITE R!_drift!_d := <<R!_drift!_d>> $ 
        WRITE $ 
        IF potential!_flag = 1 THEN WRITE  
                "The drift is the gradient of a potential." 
                ELSE WRITE "The drift appears not to be rotation-free." $ 
        WRITE ""$ 
        WRITE "The components of Ricci curvature:" $ 
        ON NERO $ WRITE Ricci:=<<Ricci>> $ OFF NERO$ 
        WRITE "------------------------------"   
        END $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
PROCEDURE Compute!_Geometry(label,Diffusion!_List) $ 
BEGIN SCALAR potential!_flag $ 
        Initialize!_tensors(Diffusion!_List) $ 
        IF Debug = 0 THEN WRITE label," Tensors initialized." $ 
        IF Debug = 0 THEN WRITE label," Computing metric tensors." $ 
        Compute!_metric() $ 
        IF Debug = 0 THEN WRITE label," Computing Christoffel symbols." $ 
        Compute!_Christoffel() $ 
        IF Debug = 0 THEN WRITE label," Computing Ricci curvature." $ 
        Compute!_Ricci() $ 
        IF Debug = 0 THEN WRITE label," Computing intrinsic drift." $ 
        Compute!_Drift() $ 
        IF Debug = 0 THEN WRITE label, 
                                " Testing for gradient-potential form." $ 
        potential!_flag := Test!_Potential() $ 
        Report!_Geometry(label, potential!_flag) $ 
        END $ 
% ------------------------------------------------ 
 
% ------------------------------------------------ 
% Compute the Ricci curvature tensor using Emery-Bakry 
% second-Gamma-operator (THIS IS SLOW!!):  
% 
ALGEBRAIC PROCEDURE Gamma(f,g) $ <<(d f)*(d g)>>/dt $ 
% 
ALGEBRAIC PROCEDURE half!_Laplace!_Beltrami(f) $ 
<<Drift d f>>/dt  
        - FOR i:=1:n!_D!_L SUM (DF(f,R!_arg(i,1))*R!_drift!_u(i,1)) $ 
% 
ALGEBRAIC PROCEDURE BiHessian(u,v) $ 
FOR i:=1:n!_D!_L SUM FOR k:=1:n!_D!_L SUM R!_g(i,k)* 
FOR j:=1:n!_D!_L SUM FOR l:=1:n!_D!_L SUM R!_g(j,l)*  
        Gamma!_ddu(i,j,u) * Gamma!_ddu(k,l,v) $ 
% 
PROCEDURE Emery!_Bakry $ 
BEGIN 
        FOR i:=1:n!_D!_L DO 
        FOR j:=i:n!_D!_L DO Ricci!_uu(i,j) := Ricci!_uu(j,i) :=  
        (         (half!_Laplace!_Beltrami R!_g(i,j))       
                - Gamma(half!_Laplace!_Beltrami R!_arg(i,1), R!_arg(j,1)) 
                - Gamma(half!_Laplace!_Beltrami R!_arg(j,1), R!_arg(i,1))             ) 
        - 
                BiHessian(i,j) $ 
        RETURN Ricci := Ricci!_uu*R!_ginv $ 
        END $ 
% ------------------------------------------------ 
OFF COMP$ 
$END$ 
 
 
